﻿using P5T.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;

namespace P5T.Controllers
{
    public class AuthController : Controller
    {
        [HttpGet]
        public ActionResult Register()
        {
            return View();
        }

        [HttpPost]
        public ActionResult Register(string username, string password, string confirm_password, 
                                     string firstname, string lastname, string middle_init, 
                                     string title, string phone, string email)
        {
            var messages = new List<String>();

            // Check that user with the same username doesn't exist.
            if (ResearcherModel.Find(username) != null)
            {
                messages.Add("A user by that name already exists. Choose another username.");
            }

            // Validate Passwords
            switch (Util.ValidatePasswordCombo(password, confirm_password))
            {
                case Util.PasswordChange.DoNotMatch:
                    messages.Add("The given passwords do not match.");
                    break;
                case Util.PasswordChange.Insufficient:
                    // TODO: Update this message.
                    messages.Add("The given password is insufficient. Passwords must contain: ");
                    break;
                default:
                    break;
            }

            // Check the phone number is valid.
            if (!Util.ValidatePhoneNumber(phone))
            {
                messages.Add("The given phone number is invalid.");
            }

            // Check the email is valid.
            if (!Util.ValidateEmail(email))
            {
                messages.Add("The given email is invalid.");
            }

            if (messages.Count == 0)
            {
                // TODO : Check if the same author already exists in the DB. 
                //        If they do, ask the user if it is a match and associate them if needed.

                // Create the necessary affiliation
                var affiliation = AffiliationModel.Create(title, phone, email);
                
                // Create the author
                char? middle_initial = null;
                if (middle_init != null && middle_init != "")
                {
                    middle_initial = middle_init.ElementAt(0);
                }
                var author = AuthorModel.Create(affiliation, firstname, lastname, middle_initial , new List<AffiliationModel>());

                // Create the researcher
                var researcher = ResearcherModel.Create(author, username, password, DateTime.Now);

                FormsAuthentication.SetAuthCookie(username, true);
                return RedirectToAction("Index", "Home");
            }
            else
            {
                ViewBag.Username = username;
                ViewBag.FirstName = firstname;
                ViewBag.LastName = lastname;
                ViewBag.MiddleInitial = middle_init;
                ViewBag.UserTitle = title;
                ViewBag.PhoneNumber = phone;
                ViewBag.Email = email;

                ViewBag.Messages = messages;

                return View();
            }
        }

        [HttpGet]
        public ActionResult Login()
        {
            return View();
        }

        [HttpPost]
        public ActionResult Login(string username, string password)
        {
            bool authValid = false;

            if (!String.IsNullOrWhiteSpace(username) && !String.IsNullOrWhiteSpace(password))
            {
                authValid = ResearcherModel.Authenticate(username, password);
            }

            if (!authValid)
            {
                ViewBag.ErrorMessage = "Invalid username or password.";
                return View();
            }
            else
            {
                FormsAuthentication.SetAuthCookie(username, true);
                return RedirectToAction("Index", "Home");
            }
        }

        public ActionResult LogOff()
        {
            FormsAuthentication.SignOut();
            return RedirectToAction("Index", "Home");
        }

    }
}
