﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using P5T.Models;

namespace P5T.Controllers
{
    [AuthFilter]
    public class InterviewController : Controller
    {
        [HttpGet]
        public ActionResult Create(int id)
        {
            ViewBag.model = ProgramModel.Find(id);
            ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
            return View("~/Views/Shared/Artifacts/Create.cshtml");
        }

        [HttpPost]
        public ActionResult Create(int id, string topic, string description, string outlet, string location, string date)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(id);
            if (program != null)
            {
                DateTime dt = DateTime.Parse(date);
                var interview = MediaInterviewModel.Create(program, description, topic, outlet, location, dt);
                MediaInterviewModel.AddAuthor(interview, currentUser.Author.Value.Id);

                MediaInterviewActivityModel.CreateCreateActivity(DateTime.Now, currentUser.Id, interview.Id); 
                return Json("{success: true}");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult Edit(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            ViewBag.model = MediaInterviewModel.Find(id);
            if (ViewBag.model != null)
            {
                ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                return View("~/Views/Shared/Artifacts/Edit.cshtml");
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult Edit(int program_id, int id, string topic, string description, string outlet, string location, string date)
        {
            var model = MediaInterviewModel.Find(id);
            var dt = DateTime.Parse(date);
            if (model != null)
            {
                model.Topic = topic;
                model.Description = description;
                model.Outlet = outlet;
                model.Location = location;
                model.Date = dt;
                MediaInterviewModel.Update(model);
                return Json("{success: true}");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddAuthor(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                    return View("~/Views/Shared/Artifacts/AddAuthor.cshtml");
                }
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddAuthor(int program_id, int id, string author)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var interview = MediaInterviewModel.Find(id);
                if (interview != null)
                {
                    var model = AuthorModel.FindOrCreate(author);
                    MediaInterviewModel.AddAuthor(interview, model.Id);

                    MediaInterviewActivityModel.CreateAddCitationActivity(DateTime.Now, currentUser.Id, interview.Id, model.ToString());
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddFile(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                    return View("~/Views/Shared/Artifacts/AddFile.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddFile(int program_id, int id, HttpPostedFileBase file, string description)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            // Read the data into a byte array.
            byte[] data = new byte[file.InputStream.Length];
            file.InputStream.Read(data, 0, data.Length);

            var model = MediaInterviewModel.Find(id);

            var attachment = AttachmentModel.Create(file.FileName, description, new System.Data.SqlTypes.SqlBinary(data), null);
            if (attachment != null)
            {
                MediaInterviewModel.AddAttachment(model, attachment.Id);

                MediaInterviewActivityModel.CreateAddFileActivity(DateTime.Now, currentUser.Id, model.Id);
                return Content("", "text/html");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddFileRevision(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.attachment = AttachmentModel.Find(id);
                    if (ViewBag.attachment != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                        return View("~/Views/Shared/Artifacts/AddFileRevision.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddFileRevision(int program_id, int artifact_id, int id, HttpPostedFileBase file, string description)
        {
            // Read the data into a byte array.
            byte[] data = new byte[file.InputStream.Length];
            file.InputStream.Read(data, 0, data.Length);

            var model = MediaInterviewModel.Find(artifact_id);

            var old_attachment = AttachmentModel.Find(id);
            if (old_attachment != null)
            {
                var attachment = AttachmentModel.Create(file.FileName, description, new System.Data.SqlTypes.SqlBinary(data), id);
                if (attachment != null)
                {
                    MediaInterviewModel.UpdateAttachment(model, old_attachment.Id, attachment.Id);
                    return Content("", "text/html");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult ViewFileRevisions(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    var attachment = AttachmentModel.Find(id);
                    if (attachment != null)
                    {
                        ViewBag.revs = AttachmentModel.FindPrevRevisions(attachment);
                        ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                        return View("~/Views/Shared/Artifacts/ViewFileRevisions.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpGet]
        public ActionResult DeleteFile(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.file = AttachmentModel.Find(id);
                    if (ViewBag.file != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                        return View("~/Views/Shared/Artifacts/DeleteFile.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoDeleteFile(int program_id, int artifact_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = MediaInterviewModel.Find(artifact_id);
                if (model != null)
                {
                    var attachment = AttachmentModel.Find(id);
                    if (attachment != null)
                    {
                        MediaInterviewModel.RemoveAttachment(model, id);

                        MediaInterviewActivityModel.CreateRemoveFileActivity(DateTime.Now, currentUser.Id, model.Id);
                        return Json("{success: true}");
                    }
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddUrl(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                    return View("~/Views/Shared/Artifacts/AddUrl.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddUrl(int program_id, int id, string url, string description)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var model = MediaInterviewModel.Find(id);
            if (model != null)
            {
                var urlModel = UrlModel.Create(url, description);
                if (url != null)
                {
                    MediaInterviewModel.AddUrl(model, urlModel.Id);

                    MediaInterviewActivityModel.CreateAddUrlActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult UpdateUrl(int program_id, int artifact_id, int id)
        {
            ViewBag.program_id = program_id;
            ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
            var url = UrlModel.Find(id);
            if (url != null)
            {
                return View("~/Views/Shared/Artifacts/UpdateUrl.cshtml", url);
            }
            return RedirectToAction("View", "Programs", new { id = program_id });
        }

        [HttpGet]
        public ActionResult DeleteUrl(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.url = UrlModel.Find(id);
                    if (ViewBag.url != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                        return View("~/Views/Shared/Artifacts/DeleteUrl.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoDeleteUrl(int program_id, int artifact_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = MediaInterviewModel.Find(artifact_id);
                if (model != null)
                {
                    MediaInterviewModel.RemoveUrl(model, id);

                    MediaInterviewActivityModel.CreateRemoveUrlActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult Delete(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                    return View("~/Views/Shared/Artifacts/ConfirmDelete.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult ConfirmDelete(int program_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = MediaInterviewModel.Find(id);
                if (model != null)
                {
                    MediaInterviewModel.Delete(id);

                    MediaInterviewActivityModel.CreateArchivedActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult RemoveAuthor(int program_id, int id, int author_id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = MediaInterviewModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.author = AuthorModel.Find(author_id);
                    ViewBag.ArtifactType = Artifacts.Type.MediaInterview;
                    return View("~/Views/Shared/Artifacts/RemoveAuthor.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoRemoveAuthor(int program_id, int id, int author_id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = MediaInterviewModel.Find(id);
                if (model != null)
                {
                    var author = AuthorModel.Find(author_id);
                    if (author != null)
                    {
                        MediaInterviewModel.RemoveAuthor(model, author_id);

                        MediaInterviewActivityModel.CreateRemoveCitationActivity(DateTime.Now, currentUser.Id, model.Id, author.ToString());
                        return Json("{success: true}");
                    }
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult History(int id)
        {
            ViewBag.specific = true;
            return View("~/Views/Programs/ViewHistory.cshtml", MediaInterviewActivityModel.FindAllByArtifactId(id));
        }
    }
}
