﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using P5T.Models;

namespace P5T.Controllers
{
    [AuthFilter]
    public class JournalArticleController : Controller
    {
        [HttpGet]
        public ActionResult Create(int id)
        {
            ViewBag.model = ProgramModel.Find(id);
            ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
            return View("~/Views/Shared/Artifacts/Create.cshtml");
        }

        [HttpPost]
        public ActionResult Create(int program_id, string description)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var journal = JournalArticleModel.Create(program, description);
                JournalArticleModel.AddAuthor(journal, currentUser.Author.Value.Id);

                JournalArticleActivityModel.CreateCreateActivity(DateTime.Now, currentUser.Id, journal.Id);
                return Json("{success: true}");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult Edit(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            ViewBag.model = JournalArticleModel.Find(id);
            if (ViewBag.model != null)
            {
                ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                return View("~/Views/Shared/Artifacts/Edit.cshtml");
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult Edit(int program_id, int id, string description)
        {
            var model = JournalArticleModel.Find(id);
            if (model != null)
            {
                model.Description = description;
                JournalArticleModel.Update(model);
                return Json("{success: true}");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddAuthor(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                    return View("~/Views/Shared/Artifacts/AddAuthor.cshtml");
                }
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddAuthor(int program_id, int id, string author)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var ja = JournalArticleModel.Find(id);
                if (ja != null)
                {
                    var model = AuthorModel.FindOrCreate(author);
                    JournalArticleModel.AddAuthor(ja, model.Id);

                    JournalArticleActivityModel.CreateAddCitationActivity(DateTime.Now, currentUser.Id, ja.Id, model.ToString());
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddFile(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                    return View("~/Views/Shared/Artifacts/AddFile.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddFile(int program_id, int id, HttpPostedFileBase file, string description)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            // Read the data into a byte array.
            byte[] data = new byte[file.InputStream.Length];
            file.InputStream.Read(data, 0, data.Length);

            var model = JournalArticleModel.Find(id);

            var attachment = AttachmentModel.Create(file.FileName, description, new System.Data.SqlTypes.SqlBinary(data), null);
            if (attachment != null)
            {
                JournalArticleModel.AddAttachment(model, attachment.Id);

                JournalArticleActivityModel.CreateAddFileActivity(DateTime.Now, currentUser.Id, model.Id);
                return Content("", "text/html");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddFileRevision(int program_id, int artifact_id, int id, bool? ajax)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.attachment = AttachmentModel.Find(id);
                    if (ViewBag.attachment != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                        return View("~/Views/Shared/Artifacts/AddFileRevision.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddFileRevision(int program_id, int artifact_id, int id, HttpPostedFileBase file, string description)
        {
            // Read the data into a byte array.
            byte[] data = new byte[file.InputStream.Length];
            file.InputStream.Read(data, 0, data.Length);

            var model = JournalArticleModel.Find(artifact_id);

            var old_attachment = AttachmentModel.Find(id);
            if (old_attachment != null)
            {
                var attachment = AttachmentModel.Create(file.FileName, description, new System.Data.SqlTypes.SqlBinary(data), id);
                if (attachment != null)
                {
                    JournalArticleModel.UpdateAttachment(model, old_attachment.Id, attachment.Id);
                    return Content("", "text/html");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult ViewFileRevisions(int program_id, int artifact_id, int id, bool? ajax)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    var attachment = AttachmentModel.Find(id);
                    if (attachment != null)
                    {
                        ViewBag.revs = AttachmentModel.FindPrevRevisions(attachment);
                        ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                        return View("~/Views/Shared/Artifacts/ViewFileRevisions.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpGet]
        public ActionResult DeleteFile(int program_id, int artifact_id, int id, bool? ajax)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.file = AttachmentModel.Find(id);
                    if (ViewBag.file != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                        return View("~/Views/Shared/Artifacts/DeleteFile.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs"); ;
        }

        [HttpPost]
        public ActionResult DoDeleteFile(int program_id, int artifact_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = JournalArticleModel.Find(artifact_id);
                if (model != null)
                {
                    var attachment = AttachmentModel.Find(id);
                    if (attachment != null)
                    {
                        JournalArticleModel.RemoveAttachment(model, id);

                        JournalArticleActivityModel.CreateRemoveFileActivity(DateTime.Now, currentUser.Id, model.Id);
                        return Json("{success: true}");
                    }
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddUrl(int program_id, int id, bool? ajax)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                    return View("~/Views/Shared/Artifacts/AddUrl.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddUrl(int program_id, int id, string url, string description)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var model = JournalArticleModel.Find(id);
            if (model != null)
            {
                var urlModel = UrlModel.Create(url, description);
                if (url != null)
                {
                    JournalArticleModel.AddUrl(model, urlModel.Id);

                    JournalArticleActivityModel.CreateAddUrlActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult UpdateUrl(int program_id, int artifact_id, int id, bool? ajax)
        {
            ViewBag.program_id = program_id;
            ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
            var url = UrlModel.Find(id);
            if (url != null)
            {
                return View("~/Views/Shared/Artifacts/UpdateUrl.cshtml", url);
            }
            return RedirectToAction("View", "Programs", new { id = program_id });
        }

        [HttpGet]
        public ActionResult DeleteUrl(int program_id, int artifact_id, int id, bool? ajax)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.url = UrlModel.Find(id);
                    if (ViewBag.url != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                        return View("~/Views/Shared/Artifacts/DeleteUrl.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoDeleteUrl(int program_id, int artifact_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = JournalArticleModel.Find(artifact_id);
                if (model != null)
                {
                    JournalArticleModel.RemoveUrl(model, id);

                    JournalArticleActivityModel.CreateRemoveUrlActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult Delete(int program_id, int id, bool? ajax)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                    return View("~/Views/Shared/Artifacts/ConfirmDelete.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult ConfirmDelete(int program_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = JournalArticleModel.Find(id);
                if (model != null)
                {
                    JournalArticleModel.Delete(id);

                    JournalArticleActivityModel.CreateArchivedActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult RemoveAuthor(int program_id, int id, int author_id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = JournalArticleModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.author = AuthorModel.Find(author_id);
                    ViewBag.ArtifactType = Artifacts.Type.JournalArticle;
                    return View("~/Views/Shared/Artifacts/RemoveAuthor.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoRemoveAuthor(int program_id, int id, int author_id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = JournalArticleModel.Find(id);
                if (model != null)
                {
                    var author = AuthorModel.Find(author_id);
                    if(author != null)
                    {
                        JournalArticleModel.RemoveAuthor(model, author_id);

                        JournalArticleActivityModel.CreateRemoveCitationActivity(DateTime.Now, currentUser.Id, model.Id, author.ToString());
                        return Json("{success: true}");
                    }
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult History(int id)
        {
            ViewBag.specific = true;
            return View("~/Views/Programs/ViewHistory.cshtml", JournalArticleActivityModel.FindAllByArtifactId(id));
        }
    }
}
