﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using P5T.Models;

namespace P5T.Controllers
{
    [AuthFilter]
    public class PresentationController : Controller
    {
        [HttpGet]
        public ActionResult Create(int id)
        {
            ViewBag.model = ProgramModel.Find(id);
            ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
            return View("~/Views/Shared/Artifacts/Create.cshtml");
        }

        [HttpPost]
        public ActionResult Create(int id, string title, string description, string type, string name, string location, string date)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(id);
            if (program != null)
            {
                DateTime dt = DateTime.Parse(date); 
                var presentation = ConferencePresentationModel.Create(program, title, description, type, name, location, dt);
                ConferencePresentationModel.AddAuthor(presentation, currentUser.Author.Value.Id);

                ConferencePresentationActivityModel.CreateCreateActivity(DateTime.Now, currentUser.Id, presentation.Id);
                return Json("{success: true}");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult Edit(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            ViewBag.model = ConferencePresentationModel.Find(id);
            if (ViewBag.model != null)
            {
                ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                return View("~/Views/Shared/Artifacts/Edit.cshtml");
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult Edit(int program_id, int id, string title, string description, string type, string name, string location, string date)
        {
            ViewBag.Id = id;
            ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
            DateTime dt = DateTime.Parse(date);
            var presentation = ConferencePresentationModel.Find(id);
            if (presentation != null)
            {
                presentation.Title = title;
                presentation.Description = description;
                presentation.Type = type;
                presentation.ConferenceName = name;
                presentation.ConferenceLocation = location;
                presentation.ConferenceDate = dt;
                ConferencePresentationModel.Update(presentation);
                return Json("{success: true}");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddAuthor(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                    return View("~/Views/Shared/Artifacts/AddAuthor.cshtml");
                }
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddAuthor(int program_id, int id, string author)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var presentation = ConferencePresentationModel.Find(id);
                if (presentation != null)
                {
                    var model = AuthorModel.FindOrCreate(author);
                    ConferencePresentationModel.AddAuthor(presentation, model.Id);

                    ConferencePresentationActivityModel.CreateAddCitationActivity(DateTime.Now, currentUser.Id, presentation.Id, model.ToString());
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddFile(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                    return View("~/Views/Shared/Artifacts/AddFile.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddFile(int program_id, int id, HttpPostedFileBase file, string description)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            // Read the data into a byte array.
            byte[] data = new byte[file.InputStream.Length];
            file.InputStream.Read(data, 0, data.Length);

            var model = ConferencePresentationModel.Find(id);

            var attachment = AttachmentModel.Create(file.FileName, description, new System.Data.SqlTypes.SqlBinary(data), null);
            if (attachment != null)
            {
                ConferencePresentationModel.AddAttachment(model, attachment.Id);

                ConferencePresentationActivityModel.CreateAddFileActivity(DateTime.Now, currentUser.Id, model.Id);
                return Content("", "text/html");
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddFileRevision(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.attachment = AttachmentModel.Find(id);
                    if (ViewBag.attachment != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                        return View("~/Views/Shared/Artifacts/AddFileRevision.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddFileRevision(int program_id, int artifact_id, int id, HttpPostedFileBase file, string description)
        {
            // Read the data into a byte array.
            byte[] data = new byte[file.InputStream.Length];
            file.InputStream.Read(data, 0, data.Length);

            var model = ConferencePresentationModel.Find(artifact_id);

            var old_attachment = AttachmentModel.Find(id);
            if (old_attachment != null)
            {
                var attachment = AttachmentModel.Create(file.FileName, description, new System.Data.SqlTypes.SqlBinary(data), id);
                if (attachment != null)
                {
                    ConferencePresentationModel.UpdateAttachment(model, old_attachment.Id, attachment.Id);
                    return Content("", "text/html");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult ViewFileRevisions(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    var attachment = AttachmentModel.Find(id);
                    if (attachment != null)
                    {
                        ViewBag.revs = AttachmentModel.FindPrevRevisions(attachment);
                        ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                        return View("~/Views/Shared/Artifacts/ViewFileRevisions.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpGet]
        public ActionResult DeleteFile(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.file = AttachmentModel.Find(id);
                    if (ViewBag.file != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                        return View("~/Views/Shared/Artifacts/DeleteFile.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoDeleteFile(int program_id, int artifact_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = ConferencePresentationModel.Find(artifact_id);
                if (model != null)
                {
                    var attachment = AttachmentModel.Find(id);
                    if (attachment != null)
                    {
                        ConferencePresentationModel.RemoveAttachment(model, id);

                        ConferencePresentationActivityModel.CreateRemoveFileActivity(DateTime.Now, currentUser.Id, model.Id);
                        return Json("{success: true}");
                    }
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult AddUrl(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                    return View("~/Views/Shared/Artifacts/AddUrl.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult AddUrl(int program_id, int id, string url, string description)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var model = ConferencePresentationModel.Find(id);
            if (model != null)
            {
                var urlModel = UrlModel.Create(url, description);
                if (url != null)
                {
                    ConferencePresentationModel.AddUrl(model, urlModel.Id);

                    ConferencePresentationActivityModel.CreateAddUrlActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult UpdateUrl(int program_id, int artifact_id, int id)
        {
            ViewBag.program_id = program_id;
            ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
            var url = UrlModel.Find(id);
            if (url != null)
            {
                return View("~/Views/Shared/Artifacts/UpdateUrl.cshtml", url);
            }
            return RedirectToAction("View", "Programs", new { id = program_id });
        }

        [HttpGet]
        public ActionResult DeleteUrl(int program_id, int artifact_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(artifact_id);
                if (ViewBag.model != null)
                {
                    ViewBag.url = UrlModel.Find(id);
                    if (ViewBag.url != null)
                    {
                        ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                        return View("~/Views/Shared/Artifacts/DeleteUrl.cshtml");
                    }
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoDeleteUrl(int program_id, int artifact_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = ConferencePresentationModel.Find(artifact_id);
                if (model != null)
                {
                    ConferencePresentationModel.RemoveUrl(model, id);

                    ConferencePresentationActivityModel.CreateRemoveUrlActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult Delete(int program_id, int id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                    return View("~/Views/Shared/Artifacts/ConfirmDelete.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult ConfirmDelete(int program_id, int id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = ConferencePresentationModel.Find(id);
                if (model != null)
                {
                    ConferencePresentationModel.Delete(id);

                    ConferencePresentationActivityModel.CreateArchivedActivity(DateTime.Now, currentUser.Id, model.Id);
                    return Json("{success: true}");
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult RemoveAuthor(int program_id, int id, int author_id)
        {
            ViewBag.program = ProgramModel.Find(program_id);
            if (ViewBag.program != null)
            {
                ViewBag.model = ConferencePresentationModel.Find(id);
                if (ViewBag.model != null)
                {
                    ViewBag.author = AuthorModel.Find(author_id);
                    ViewBag.ArtifactType = Artifacts.Type.ConferencePresentation;
                    return View("~/Views/Shared/Artifacts/RemoveAuthor.cshtml");
                }
                return RedirectToAction("View", "Programs", new { id = program_id });
            }
            return RedirectToAction("List", "Programs");
        }

        [HttpPost]
        public ActionResult DoRemoveAuthor(int program_id, int id, int author_id)
        {
            var currentUser = RouteData.DataTokens["user"] as ResearcherModel;

            var program = ProgramModel.Find(program_id);
            if (program != null)
            {
                var model = ConferencePresentationModel.Find(id);
                if (model != null)
                {
                    var author = AuthorModel.Find(author_id);
                    if (author != null)
                    {
                        ConferencePresentationModel.RemoveAuthor(model, author_id);

                        ConferencePresentationActivityModel.CreateRemoveCitationActivity(DateTime.Now, currentUser.Id, model.Id, author.ToString());
                        return Json("{success: true}");
                    }
                }
            }
            return HttpNotFound();
        }

        [HttpGet]
        public ActionResult History(int id)
        {
            ViewBag.specific = true;
            return View("~/Views/Programs/ViewHistory.cshtml", ConferencePresentationActivityModel.FindAllByArtifactId(id));
        }
    }
}
