﻿using P5T.Models;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Security.Cryptography;
using System.Text;
using System.Globalization;
using System.Web.Configuration;

namespace P5T
{
    public class DatabaseFacade
    {
        public static SqlConnection Open()
        {
            var rootWebConf = WebConfigurationManager.OpenWebConfiguration("/");
            System.Configuration.ConnectionStringSettings connString;
            if (rootWebConf.ConnectionStrings.ConnectionStrings.Count > 0)
            {
                connString = rootWebConf.ConnectionStrings.ConnectionStrings["P5TConnection"];
                if (connString != null)
                {
                    return new SqlConnection(connString.ConnectionString);
                }
            }
            throw new SystemException("WebConfig not found or missing connection string P5TConnection");
        }

        public delegate bool TransactionDelegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd);
        public static void ExecuteTx(TransactionDelegate functor)
        {
            using (var conn = DatabaseFacade.Open())
            {
                conn.Open();
                using (var tx = conn.BeginTransaction())
                {
                    try
                    {
                        bool commit = false;
                        using (var cmd = conn.CreateCommand())
                        {
                            cmd.Transaction = tx;
                            commit = functor(conn, tx, cmd);
                        }

                        if (commit)
                            tx.Commit();
                        else
                            tx.Rollback();
                    }
                    catch (Exception e1)
                    {
                        try
                        {
                            tx.Rollback();
                        }
                        catch (Exception e2)
                        {
                            throw new Exception(e2.Message, e1);
                        }
                        throw e1;
                    }
                }
                conn.Close();
            }
        }

        public delegate T TypedTransactionDelegate<T>(SqlConnection conn, SqlTransaction tx, SqlCommand cmd);
        public static T ExecuteTypedTx<T>(TypedTransactionDelegate<T> functor)
        {
            T commit;
            using (var conn = DatabaseFacade.Open())
            {
                conn.Open();
                using (var tx = conn.BeginTransaction())
                {
                    try
                    {
                        using (var cmd = conn.CreateCommand())
                        {
                            cmd.Transaction = tx;
                            commit = functor(conn, tx, cmd);
                        }

                        if (commit != null)
                            tx.Commit();
                        else
                            tx.Rollback();
                    }
                    catch (Exception e1)
                    {
                        try
                        {
                            tx.Rollback();
                        }
                        catch (Exception e2)
                        {
                            throw new Exception(e2.Message, e1);
                        }
                        throw e1;
                    }
                }
                conn.Close();
            }
            return commit;
        }

        public static String StringOrNull(SqlDataReader rdr, int column)
        {
            if (rdr.IsDBNull(column))
                return null;
            else
                return rdr.GetString(column);
        }

        public static DateTime? DateTimeOrNull(SqlDataReader rdr, int column)
        {
            if(rdr.IsDBNull(column))
                return null;
            else
                return rdr.GetDateTime(column);
        }
    }
}
