﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace P5T
{
    public class Lazy<T> 
        where T : class
    {
        public delegate T LazyDelegate();

        public Lazy(T data, object raw_value = null)
        {
            Data = data;
            RawValue = raw_value;
        }

        public Lazy(LazyDelegate functor, object raw_value)
        {
            Data = null;
            RawValue = raw_value;
            Functor = functor;
        }

        public bool IsLoaded()
        {
            return Data != null;
        }

        private void Load()
        {
            if (Data == null && Functor != null)
            {
                Data = Functor();
            }
        }

        public T Value
        {
            get 
            {
                Load();
                return Data;
            }

            set 
            {
                Data = value;
            }
        }

        private T Data;
        public object RawValue;
        private LazyDelegate Functor;
    }
}