﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace P5T
{
    public class LazyList<T> : IList<T>, ICollection<T>
    {
        public delegate List<T> LoaderDelegate();

        public LazyList(List<T> content)
        {
            Functor = null;
            Data = content;
        }

        public LazyList(LoaderDelegate functor, object raw_value)
        {
            Functor = functor;
            Data = null;
            RawValue = raw_value;
        }

        public bool IsLoaded()
        {
            return Data != null;
        }

        private void Load()
        {
            if (Data == null && Functor != null)
            {
                Data = Functor();
            }
        }

        public int IndexOf(T item)
        {
            Load();
            return Data.IndexOf(item);
        }

        public void Insert(int index, T item)
        {
            Load();
            Data.Insert(index, item);
        }

        public void RemoveAt(int index)
        {
            Load();
            Data.RemoveAt(index);
        }

        public T this[int index]
        {
            get
            {
                Load();
                return Data[index];
            }
            set
            {
                Load();
                Data[index] = value;
            }
        }

        public void Add(T item)
        {
            Load();
            Add(item);
        }

        public void Clear()
        {
            if (Data == null)
            {
                Data = new List<T>();
            }
            else
            {
                Data.Clear();
            }
        }

        public bool Contains(T item)
        {
            Load();
            return Data.Contains(item);
        }

        public void CopyTo(T[] array, int arrayIndex)
        {
            Load();
            Data.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get
            {
                Load();
                return Data.Count;
            }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(T item)
        {
            Load();
            return Data.Remove(item);
        }

        public IEnumerator<T> GetEnumerator()
        {
            Load();
            return Data.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            Load();
            return Data.GetEnumerator();
        }  

        private LoaderDelegate Functor;
        public object RawValue;
        private List<T> Data;
    }
}