﻿using System;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Data;

namespace P5T.Models
{
    public class AffiliationModel
    {
        public int Id;
        public string Title;
        public string PhoneNumber;
        public string EmailAddress;

        public AffiliationModel(int id, string title, string phone, string email)
        {
            Id = id;
            Title = title;
            PhoneNumber = phone;
            EmailAddress = email;
        }

        public static AffiliationModel Create(string title, string phone, string email)
        {
            // Create a new Affiliation Model and return it.
            return DatabaseFacade.ExecuteTypedTx<AffiliationModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO Affiliation ([title], [phone_number], [email_address]) OUTPUT INSERTED.affiliation_id VALUES (@Title, @PhoneNumber, @EmailAddress)";
                cmd.Parameters.Add("@Title", SqlDbType.NVarChar, 128).Value = title;
                cmd.Parameters.Add("@PhoneNumber", SqlDbType.NChar, 11).Value = phone;
                cmd.Parameters.Add("@EmailAddress", SqlDbType.NVarChar, 128).Value = email;
                cmd.Prepare();

                return new AffiliationModel((Int32)cmd.ExecuteScalar(), title, phone, email);
            });
        }

        public static AffiliationModel Find(int id)
        {
            // Find the necessary AffiliationModel and return it.
            return DatabaseFacade.ExecuteTypedTx<AffiliationModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT title, phone_number, email_address FROM Affiliation WHERE affiliation_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();

                using (var reader = cmd.ExecuteReader())
                {
                    if (reader.Read())
                    {
                        return new AffiliationModel(id, 
                            reader.GetString(reader.GetOrdinal("title")), 
                            reader.GetString(reader.GetOrdinal("phone_number")),
                            reader.GetString(reader.GetOrdinal("email_address")));
                    }
                }

                return null;
            });
        }

        public static void Update(AffiliationModel model)
        {
            // Update the given Affiliation Model with the necessary internal values.
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE Affiliation SET title = @Title, phone_number = @PhoneNumber, email_address = @EmailAddress WHERE affiliation_id = @Id";
                cmd.Parameters.Add("@Title", SqlDbType.NVarChar, 128).Value = model.Title;
                cmd.Parameters.Add("@PhoneNumber", SqlDbType.NChar, 11).Value = model.PhoneNumber;
                cmd.Parameters.Add("@EmailAddress", SqlDbType.NVarChar, 128).Value = model.EmailAddress;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void Delete(int id)
        {
            // Delete the given Affiliation Model from the DB.
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM Affiliation WHERE affiliation_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }
    }
}
