using System;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;

namespace P5T.Models
{
    public class ConferencePresentationModel
    {
        public int Id;
        public Lazy<ProgramModel> Program;
        public string Description;
        public string Title;
        public string Type;
        public string ConferenceName;
        public string ConferenceLocation;
        public DateTime ConferenceDate;
        public DateTime? ArchivalDate;

        public LazyList<AuthorModel> Authors;
        public LazyList<AttachmentModel> Attachments;
        public LazyList<UrlModel> Urls;


        public ConferencePresentationModel() { }


        public ConferencePresentationModel(int id, ProgramModel program, string title, string description, string type, string conference_name, string conference_location, DateTime conference_date, DateTime? ArchivalDate)
        {
            Id = id;
            Program = new Lazy<ProgramModel>(program);
            Description = description;
            Title = title;
            Type = type;
            ConferenceName = conference_name;
            ConferenceLocation = conference_location;
            ConferenceDate = conference_date;

            InitializeLazyLists();
        }

        public ConferencePresentationModel(int id, int program_id, string title, string description, string type, string conference_name, string conference_location, DateTime conference_date, DateTime? ArchivalDate)
        {
            Id = id;
            Description = description;
            Title = title;
            Type = type;
            ConferenceName = conference_name;
            ConferenceLocation = conference_location;
            ConferenceDate = conference_date;

            Program = new Lazy<ProgramModel>(delegate()
            {
                return ProgramModel.Find(program_id);
            }, program_id);

            InitializeLazyLists();
        }

        public String GetMetadata()
        {
            String result = String.Format("Description: {0}\nTitle: {1}\nType: {2}\nConference Name: {3}\nConference Location: {4}\nConference Date: {5}\nAuthors:\n",
                Description, Title, Type, ConferenceName, ConferenceLocation, ConferenceDate);
            foreach (var s in Authors)
            {
                result += "\t" + s.ToString();
            }
            return result;
        }

        private void InitializeLazyLists()
        {
            Authors = new LazyList<AuthorModel>(delegate()
            {
                var result = new List<AuthorModel>();
                // TODO : Use Id to get the authors from the database and return it.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT author_id FROM ConferencePresentationAuthor WHERE presentation_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Id;
                    cmd.Prepare();

                    using (var reader = cmd.ExecuteReader())
                    {
                        while (reader.Read())
                        {
                            var author = AuthorModel.Find(reader.GetInt32(reader.GetOrdinal("author_id")));
                            if (author != null)
                            {
                                result.Add(author);
                            }
                        }
                    }
                    return true;
                });
                return result;
            }, Id);

            Attachments = new LazyList<AttachmentModel>(delegate()
            {
                var result = new List<AttachmentModel>();
                // TODO : Use Id to get the attachments from the DB and return it.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT attachment_id FROM ConferencePresentationAttachment WHERE presentation_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Id;
                    cmd.Prepare();

                    using (var reader = cmd.ExecuteReader())
                    {
                        while (reader.Read())
                        {
                            var attachment = AttachmentModel.Find(reader.GetInt32(reader.GetOrdinal("attachment_id")));
                            if (attachment != null)
                            {
                                result.Add(attachment);
                            }
                        }
                    }
                    return true;
                });
                return result;
            }, Id);

            Urls = new LazyList<UrlModel>(delegate()
            {
                var result = new List<UrlModel>();
                // TODO : Use Id to get the Urls from the DB and return it.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT url_id FROM ConferencePresentationUrl WHERE presentation_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Id;
                    cmd.Prepare();

                    using (var reader = cmd.ExecuteReader())
                    {
                        while (reader.Read())
                        {
                            var url = UrlModel.Find(reader.GetInt32(reader.GetOrdinal("url_id")));
                            if (url != null)
                            {
                                result.Add(url);
                            }
                        }
                    }
                    return true;
                });
                return result;
            }, Id);
        }

        public static ConferencePresentationModel Create(ProgramModel program, string title, string description, string type, string name, string location, DateTime date)
        {
            // TODO : Create a new ConferencePresentationModel and return it.
            return DatabaseFacade.ExecuteTypedTx<ConferencePresentationModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO ConferencePresentation ([program_id], [presentation_title], [presentation_description], [presentation_type], [conference_name], [conference_location], [conference_date]) OUTPUT INSERTED.presentation_id VALUES (@ProgId, @Title, @Description, @Type, @Name, @Location, @Date)";
                cmd.Parameters.Add("@ProgId", SqlDbType.Int).Value = program.Id;
                cmd.Parameters.Add("@Title", SqlDbType.NVarChar, 50).Value = title;
                cmd.Parameters.Add("@Description", SqlDbType.NVarChar, 256).Value = description;
                cmd.Parameters.Add("@Type", SqlDbType.NVarChar, 64).Value = type;
                cmd.Parameters.Add("@Name", SqlDbType.NVarChar, 128).Value = name;
                cmd.Parameters.Add("@Location", SqlDbType.NVarChar, 128).Value = location;
                cmd.Parameters.Add("@Date", SqlDbType.DateTime).Value = date;
                cmd.Prepare();
                return new ConferencePresentationModel((Int32)cmd.ExecuteScalar(), program.Id, title, description, type, name, location, date, null);
            });
        }

        public static ConferencePresentationModel Find(int id)
        {
            // TODO : Find the ConferencePresentationModel with the given ID and return it.
            return DatabaseFacade.ExecuteTypedTx<ConferencePresentationModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT program_id, presentation_title, presentation_description, presentation_type, conference_name, conference_location, conference_date, archival_date FROM ConferencePresentation WHERE presentation_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();

                using (var reader = cmd.ExecuteReader())
                {
                    if (reader.Read())
                    {
                        return new ConferencePresentationModel(id,
                            reader.GetInt32(reader.GetOrdinal("program_id")),
                            reader.GetString(reader.GetOrdinal("presentation_title")),
                            reader.GetString(reader.GetOrdinal("presentation_description")),
                            reader.GetString(reader.GetOrdinal("presentation_type")),
                            reader.GetString(reader.GetOrdinal("conference_name")),
                            reader.GetString(reader.GetOrdinal("conference_location")),
                            reader.GetDateTime(reader.GetOrdinal("conference_date")),
                            DatabaseFacade.DateTimeOrNull(reader, reader.GetOrdinal("archival_date")));
                    }
                }

                return null;
            });
        }

        public static void Update(ConferencePresentationModel model)
        {
            // TODO : Update only the fields in the Grant table associated with the given model.
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE ConferencePresentation SET presentation_title = @Title, presentation_description = @Description, presentation_type = @Type, conference_name = @Name, conference_location = @Location, conference_date = @Date WHERE presentation_id = @Id";
                cmd.Parameters.Add("@Title", SqlDbType.NVarChar, 50).Value = model.Title;
                cmd.Parameters.Add("@Description", SqlDbType.NVarChar, 256).Value = model.Description;
                cmd.Parameters.Add("@Type", SqlDbType.NVarChar, 64).Value = model.Type;
                cmd.Parameters.Add("@Name", SqlDbType.NVarChar, 128).Value = model.ConferenceName;
                cmd.Parameters.Add("@Location", SqlDbType.NVarChar, 128).Value = model.ConferenceLocation;
                cmd.Parameters.Add("@Date", SqlDbType.DateTime).Value = model.ConferenceDate;
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static void AddAuthor(ConferencePresentationModel model, int author)
        {
            // TODO : Add the Author to the given ConferencePresentationModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT COUNT(*) FROM ConferencePresentationAuthor WHERE presentation_id=@PresId AND author_id=@AuthId";
                cmd.Parameters.Add("@PresId", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@AuthId", SqlDbType.Int).Value = author;
                cmd.Parameters.Add("@AuthOrder", SqlDbType.Int).Value = 0;
                cmd.Prepare();

                if (0 == (Int32)cmd.ExecuteScalar())
                {
                    cmd.CommandText = "INSERT INTO ConferencePresentationAuthor ([presentation_id], [author_id], [author_order]) OUTPUT INSERTED.presentation_id VALUES (@PresId, @AuthId, @AuthOrder)";
                    cmd.Prepare();
                    return cmd.ExecuteNonQuery() == 1;
                }
                return false;
            });
        }

        public static void RemoveAuthor(ConferencePresentationModel model, int author)
        {
            // TODO : Remove the Author from the given ConferencePresentationModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM ConferencePresentationAuthor WHERE presentation_id = @ModelId AND author_id = @Id";
                cmd.Parameters.Add("@ModelId", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = author;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static void AddAttachment(ConferencePresentationModel model, int attachment)
        {
            // TODO : Add the Attachment to the given ConferencePresentationModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO ConferencePresentationAttachment ([presentation_id], [attachment_id], [attachment_order]) OUTPUT INSERTED.presentation_id VALUES (@PresId, @AttId, @AttOrder)";
                cmd.Parameters.Add("@PresId", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@AttId", SqlDbType.Int).Value = attachment;
                cmd.Parameters.Add("@AttOrder", SqlDbType.Int).Value = 0;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static void UpdateAttachment(ConferencePresentationModel model, int old_attachment, int new_attachment)
        {
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE ConferencePresentationAttachment SET attachment_id = @NewAttachmentId WHERE presentation_id = @Id AND attachment_id = @OldAttachmentId";
                cmd.Parameters.Add("@NewAttachmentId", SqlDbType.Int).Value = new_attachment;
                cmd.Parameters.Add("@OldAttachmentId", SqlDbType.Int).Value = old_attachment;
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void RemoveAttachment(ConferencePresentationModel model, int attachment)
        {
            // TODO : Remove the Attachment from the given ConferencePresentationModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM ConferencePresentationAttachment WHERE attachment_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = attachment;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static void AddUrl(ConferencePresentationModel model, int url)
        {
            // TODO : Add the URL to the given ConferencePresentationModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO ConferencePresentationUrl ([presentation_id], [url_id], [url_order]) OUTPUT INSERTED.presentation_id VALUES (@PresId, @UrlId, @UrlOrder)";
                cmd.Parameters.Add("@PresId", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@UrlId", SqlDbType.Int).Value = url;
                cmd.Parameters.Add("@UrlOrder", SqlDbType.Int).Value = 0;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static void RemoveUrl(ConferencePresentationModel model, int url)
        {
            // TODO : Remove the URL from the given ConferencePresentationModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM ConferencePresentationUrl WHERE url_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = url;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static void Delete(int id)
        {
            // TODO : Delete the given ConferencePresentationModel from the DB.
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE ConferencePresentation SET archival_date = CONVERT(date, GETDATE()) WHERE presentation_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static LazyList<ConferencePresentationModel> FindByProgram(int program_id, bool archived = false)
        {
            return new LazyList<ConferencePresentationModel>(delegate()
            {
                var result = new List<ConferencePresentationModel>();

                // Get the list of Conference Presentations from the DB.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT presentation_id, presentation_title, presentation_description, presentation_type, conference_name, conference_location, conference_date, archival_date FROM ConferencePresentation WHERE program_id = @ProgramId AND archival_date IS " + (archived ? "NOT " : "") + "NULL";
                    cmd.Parameters.Add("@ProgramId", SqlDbType.Int).Value = program_id;
                    cmd.Prepare();

                    using (var rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            result.Add(new ConferencePresentationModel(rdr.GetInt32(rdr.GetOrdinal("presentation_id")),
                                            program_id,
                                            rdr.GetString(rdr.GetOrdinal("presentation_title")),
                                            rdr.GetString(rdr.GetOrdinal("presentation_description")),
                                            rdr.GetString(rdr.GetOrdinal("presentation_type")),
                                            rdr.GetString(rdr.GetOrdinal("conference_name")),
                                            rdr.GetString(rdr.GetOrdinal("conference_location")),
                                            rdr.GetSqlDateTime(rdr.GetOrdinal("conference_date")).Value,
                                            DatabaseFacade.DateTimeOrNull(rdr, rdr.GetOrdinal("archival_date"))));
                        }
                    }
                    return true;
                });

                return result;
            }, program_id);
        }

        public static LazyList<ConferencePresentationModel> FindByAuthor(int id)
        {
            return new LazyList<ConferencePresentationModel>(delegate()
            {
                var result = new List<ConferencePresentationModel>();
                // Use Id to get the Journals from the DB and return them.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT presentation_id FROM ConferencePresentationAuthor WHERE author_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                    cmd.Prepare();

                    using (var rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            var article = ConferencePresentationModel.Find(rdr.GetInt32(rdr.GetOrdinal("presentation_id")));
                            if (article != null)
                            {
                                result.Add(article);
                            }
                        }
                    }

                    return true;
                });

                return result;
            }, id);
        }
    }
}