using System;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Text;
using System.Web.Configuration;
using System.Globalization;
using System.Data;

namespace P5T.Models
{
    public class OtherModel
    {
        public int Id;
        public Lazy<ProgramModel> Program;
        public string Description;
        public DateTime? ArchivalDate;

        public LazyList<AuthorModel> Authors;
        public LazyList<AttachmentModel> Attachments;
        public LazyList<UrlModel> Urls;

        public OtherModel(int id, ProgramModel program, string description, DateTime? ArchivalDate)
        {
            Id = id;
            Program = new Lazy<ProgramModel>(program);
            Description = description;

            InitializeLazyLists();
        }

        public OtherModel(int id, int program_id, string description, DateTime? ArchivalDate)
        {
            Id = id;
            Description = description;

            Program = new Lazy<ProgramModel>(delegate()
            {
                // use program_id to get the Program from the DB and return it.
                return DatabaseFacade.ExecuteTypedTx<ProgramModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    return ProgramModel.Find(program_id);
                });
            }, program_id);

            InitializeLazyLists();
        }

        public String GetMetadata()
        {
            String result = String.Format("Description: {0}\n Authors: \n", Description);
            foreach (var s in Authors)
            {
                result += "\t" + s.ToString();
            }
            return result;
        }

        private void InitializeLazyLists()
        {
            Authors = new LazyList<AuthorModel>(delegate()
            {
                var result = new List<AuthorModel>();
                // TODO : Use Id to get the authors from the database and return it.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT author_id FROM OtherAuthor WHERE other_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Id;
                    cmd.Prepare();

                    using (var rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            var author = AuthorModel.Find(rdr.GetInt32(rdr.GetOrdinal("author_id")));
                            if (author != null)
                            {
                                result.Add(author);
                            }
                        }
                    }

                    return true;
                });
                return result;
            }, Id);

            Attachments = new LazyList<AttachmentModel>(delegate()
            {
                var result = new List<AttachmentModel>();
                // TODO : Use Id to get the attachments from the DB and return it.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT attachment_id FROM OtherAttachment WHERE other_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Id;
                    cmd.Prepare();

                    using (var rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            var attachment = AttachmentModel.Find(rdr.GetInt32(rdr.GetOrdinal("attachment_id")));
                            if (attachment != null)
                            {
                                result.Add(attachment);
                            }
                        }
                    }

                    return true;
                });
                return result;
            }, Id);

            Urls = new LazyList<UrlModel>(delegate()
            {
                var result = new List<UrlModel>();
                // TODO : Use Id to get the Urls from the DB and return it.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT url_id FROM OtherUrl WHERE other_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = Id;
                    cmd.Prepare();

                    using (var rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            var url = UrlModel.Find(rdr.GetInt32(rdr.GetOrdinal("url_id")));
                            if (url != null)
                            {
                                result.Add(url);
                            }
                        }
                    }

                    return true;
                });
                return result;
            }, Id);
        }

        public static OtherModel Create(ProgramModel program, string description)
        {
            // Create a new OtherModel and return it.
            return DatabaseFacade.ExecuteTypedTx<OtherModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO Other (program_id, other_description) OUTPUT INSERTED.other_id VALUES (@ProgramId, @Description)";
                cmd.Parameters.Add("@Description", SqlDbType.NVarChar, 128).Value = description;
                cmd.Parameters.Add("@ProgramId", SqlDbType.Int).Value = program.Id;
                cmd.Prepare();

                return new OtherModel((Int32)cmd.ExecuteScalar(), program, description, null);
            });
        }

        public static OtherModel Find(int id)
        {
            return DatabaseFacade.ExecuteTypedTx<OtherModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT program_id, other_description, archival_date FROM Other WHERE other_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();

                using (var rdr = cmd.ExecuteReader())
                {
                    if (rdr.Read())
                    {
                        return new OtherModel(id,
                            rdr.GetInt32(rdr.GetOrdinal("program_id")),
                            rdr.GetString(rdr.GetOrdinal("other_description")),
                            DatabaseFacade.DateTimeOrNull(rdr, rdr.GetOrdinal("archival_date")));
                    }
                }
                return null;
            });
        }

        public static void Update(OtherModel model)
        {
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE Other SET other_description = @Description WHERE other_id = @Id";
                cmd.Parameters.Add("@Description", SqlDbType.NVarChar, 128).Value = model.Description;
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void AddAuthor(OtherModel model, int author)
        {
            // Add the Author to the given OtherModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT COUNT(*) FROM OtherAuthor WHERE other_id=@Id AND author_id=@Author";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@Author", SqlDbType.Int).Value = author;
                cmd.Parameters.Add("@AuthOrder", SqlDbType.Int).Value = 0;
                cmd.Prepare();

                if (0 == (Int32)cmd.ExecuteScalar())
                {
                    cmd.CommandText = "INSERT INTO OtherAuthor (other_id, author_id, author_order) VALUES (@Id, @Author, @AuthOrder)";
                    cmd.Prepare();
                    return cmd.ExecuteNonQuery() > 0;
                }
                return false;
            });
        }

        public static void RemoveAuthor(OtherModel model, int author)
        {
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM OtherAuthor WHERE other_id = @ModelId AND author_id = @Id";
                cmd.Parameters.Add("@ModelId", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = author;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void AddAttachment(OtherModel model, int attachment)
        {
            // Add the Attachment to the given OtherModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO OtherAttachment (other_id, attachment_id, attachment_order) VALUES (@Id, @Attachment, @Order)";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@Attachment", SqlDbType.Int).Value = attachment;
                cmd.Parameters.Add("@Order", SqlDbType.Int).Value = 0;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void UpdateAttachment(OtherModel model, int old_attachment, int new_attachment)
        {
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE OtherAttachment SET attachment_id = @NewAttachmentId WHERE other_id = @Id AND attachment_id = @OldAttachmentId";
                cmd.Parameters.Add("@NewAttachmentId", SqlDbType.Int).Value = new_attachment;
                cmd.Parameters.Add("@OldAttachmentId", SqlDbType.Int).Value = old_attachment;
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void RemoveAttachment(OtherModel model, int attachment)
        {
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM OtherAttachment WHERE attachment_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = attachment;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void AddUrl(OtherModel model, int url)
        {
            // Add the URL to the given OtherModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO OtherUrl (other_id, url_id, url_order) VALUES (@Id, @Url, @Order)";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@Url", SqlDbType.Int).Value = url;
                cmd.Parameters.Add("@Order", SqlDbType.Int).Value = 0;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void RemoveUrl(OtherModel model, int url)
        {
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM OtherUrl WHERE url_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = url;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void Delete(int id)
        {
            // Delete the given OtherModel from the DB.
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE Other SET archival_date = CONVERT(date, GETDATE()) WHERE other_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();

                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static LazyList<OtherModel> FindByProgram(int program_id, bool archived = false)
        {
            return new LazyList<OtherModel>(delegate()
            {
                var result = new List<OtherModel>();

                // Get the list of others from the DB.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT other_id, other_description, archival_date FROM Other WHERE program_id = @ProgramId AND archival_date IS " + (archived ? "NOT " : "") + "NULL";
                    cmd.Parameters.Add("@ProgramId", SqlDbType.Int).Value = program_id;
                    cmd.Prepare();

                    using (var rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            result.Add(new OtherModel(rdr.GetInt32(rdr.GetOrdinal("other_id")),
                                            program_id, 
                                            rdr.GetString(rdr.GetOrdinal("other_description")),
                                            DatabaseFacade.DateTimeOrNull(rdr, rdr.GetOrdinal("archival_date"))));
                        }
                    }

                    return true;
                });

                return result;
            }, program_id);
        }

        public static LazyList<OtherModel> FindByAuthor(int id)
        {
            return new LazyList<OtherModel>(delegate()
            {
                var result = new List<OtherModel>();
                // Use Id to get the Journals from the DB and return them.
                DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
                {
                    cmd.CommandText = "SELECT other_id FROM OtherAuthor WHERE author_id = @Id";
                    cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                    cmd.Prepare();

                    using (var rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            var article = OtherModel.Find(rdr.GetInt32(rdr.GetOrdinal("other_id")));
                            if (article != null)
                            {
                                result.Add(article);
                            }
                        }
                    }

                    return true;
                });

                return result;
            }, id);
        }
    }
}