﻿using System;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;

namespace P5T.Models
{
    public class ProgramModel
    {
        public int Id;
        public string Name;
        public string Description;
        public DateTime? ArchivalDate;

        public LazyList<ResearchArticleModel> ResearchArticles;
        public LazyList<ResearchArticleModel> ArchivedResearchArticles;
        public LazyList<JournalArticleModel> JournalArticles;
        public LazyList<JournalArticleModel> ArchivedJournalArticles;
        public LazyList<ConferencePresentationModel> ConferencePresentations;
        public LazyList<ConferencePresentationModel> ArchivedConferencePresentations;
        public LazyList<MediaInterviewModel> MediaInterviews;
        public LazyList<MediaInterviewModel> ArchivedMediaInterviews;
        public LazyList<GrantModel> Grants;
        public LazyList<GrantModel> ArchivedGrants;
        public LazyList<OtherModel> Others;
        public LazyList<OtherModel> ArchivedOthers;
        public LazyList<ActivityModel> Activity;

        public ProgramModel(int id, string name, string description, DateTime? archivalDate)
        {
            Id = id;
            Name = name;
            Description = description;

            ResearchArticles = ResearchArticleModel.FindByProgram(id, false);
            ArchivedResearchArticles = ResearchArticleModel.FindByProgram(id, true);
            
            JournalArticles = JournalArticleModel.FindByProgram(id, false);
            ArchivedJournalArticles = JournalArticleModel.FindByProgram(id, true);

            ConferencePresentations = ConferencePresentationModel.FindByProgram(id, false);
            ArchivedConferencePresentations = ConferencePresentationModel.FindByProgram(id, true);

            MediaInterviews = MediaInterviewModel.FindByProgram(id, false);
            ArchivedMediaInterviews = MediaInterviewModel.FindByProgram(id, true);

            Grants = GrantModel.FindByProgram(id, false);
            ArchivedGrants = GrantModel.FindByProgram(id, true);

            Others = OtherModel.FindByProgram(id, false);
            ArchivedOthers = OtherModel.FindByProgram(id, true);

            ArchivalDate = archivalDate;
        }

        public static ProgramModel Create(string name, string description)
        {
            // Create a new ProgramModel and return it.
            return DatabaseFacade.ExecuteTypedTx<ProgramModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO Program (name, program_description) OUTPUT INSERTED.program_id VALUES (@Name, @Description)";
                cmd.Parameters.Add("@Name", SqlDbType.NVarChar, 128).Value = name;
                cmd.Parameters.Add("@Description", SqlDbType.NVarChar, 128).Value = description;
                cmd.Prepare();
                return new ProgramModel((Int32)cmd.ExecuteScalar(), name, description, null);
            });
        }

        public static List<ProgramModel> FindAll(bool archived=false)
        {
            var result = new List<ProgramModel>();

            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT program_id, name, program_description, archival_date FROM Program WHERE archival_date IS " + (archived ? "NOT " : "") + "NULL";

                using (var rdr = cmd.ExecuteReader())
                {
                    while (rdr.Read())
                    {
                        result.Add(new ProgramModel(rdr.GetInt32(rdr.GetOrdinal("program_id")),
                            rdr.GetString(rdr.GetOrdinal("name")),
                            rdr.GetString(rdr.GetOrdinal("program_description")),
                            DatabaseFacade.DateTimeOrNull(rdr, rdr.GetOrdinal("archival_date"))));
                    }
                }

                return true;
            });

            return result;
        }

        public static ProgramModel Find(int id)
        {
            // TODO : Find a ProgramModel by id and return it
            return DatabaseFacade.ExecuteTypedTx<ProgramModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT name, program_description, archival_date FROM Program WHERE program_id = @ProgramId";
                cmd.Parameters.Add("@ProgramId", SqlDbType.Int).Value = id;
                cmd.Prepare();

                using (var rdr = cmd.ExecuteReader())
                {
                    if(rdr.Read())
                    {
                        return new ProgramModel(id,
                            rdr.GetString(rdr.GetOrdinal("name")),
                            rdr.GetString(rdr.GetOrdinal("program_description")),
                            DatabaseFacade.DateTimeOrNull(rdr, rdr.GetOrdinal("archival_date")));
                    }
                    else
                    {
                        return null;
                    }
                }
            });
        }

        public static void Update(ProgramModel model)
        {
            // Update the ProgramModel
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE Program SET name = @Name, program_description = @Desc WHERE program_id = @ProgramId";
                cmd.Parameters.Add("@Name", SqlDbType.NVarChar, 128).Value = model.Name;
                cmd.Parameters.Add("@Desc", SqlDbType.NVarChar, 128).Value = model.Description;
                cmd.Parameters.Add("@ProgramId", SqlDbType.Int).Value = model.Id;
                cmd.Prepare();
                return cmd.ExecuteNonQuery() > 0;
            });
        }

        public static void Delete(int id)
        {
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE Program SET archival_date = CONVERT(date, GETDATE()) WHERE program_id = @ProgramId";
                cmd.Parameters.Add("@ProgramId", SqlDbType.Int).Value = id;
                cmd.Prepare();
                return cmd.ExecuteNonQuery() > 0;
            });
        }
    }
}