﻿using System;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Text;
using System.Security.Cryptography;
using System.Web.Configuration;
using System.Globalization;
using System.Data;
using System.Text.RegularExpressions;

namespace P5T.Models
{
    public class SearchModel
    {
        public String Title;

        // Complete lists of models, to be generated when page loads instead of during individual queries
        private List<ResearcherModel> allResearchers;
        private List<ProgramModel> allPrograms;

        // The lists that we will be drawing our results from on the actual Search page
        private List<ResearcherModel> researcherResult;
        private List<ProgramModel> programResult;

        public SearchModel()
        {
            // Set title of page
            Title = "Search";

            // Generate lists to use in later funcs
            allResearchers = ResearcherModel.FindAll();
            allPrograms = ProgramModel.FindAll();
        }

        public List<ResearcherModel> getResearcherResult()
        {
            if (researcherResult != null)
                return researcherResult;

            return new List<ResearcherModel>();
        }

        public List<ProgramModel> getProgramResult()
        {
            if (programResult != null)
                return programResult;

            return new List<ProgramModel>();
        }

        /*
         * The search function that we call to process all searches across the database
         */
        public void SearchQuery(string query)
        {
            if (String.IsNullOrWhiteSpace(query))
                // If we load the page without searching for something...
                return;

            // Everything will be searched with an untyped search. Search typing will not be implemented.
            string[] nontypedArray = Regex.Split(query, ", ");

            researcherResult = FindResearchersBy(nontypedArray);
            programResult = FindProgramsBy(nontypedArray);
        }

        /*
         * Invariant-culture comparison of two strings.
         */
        private bool StringContains(string s1, string s2)
        {
            return CultureInfo.InvariantCulture.CompareInfo.IndexOf(s1, s2, CompareOptions.IgnoreCase) >= 0;
        }

        /*
         * Retrieves navigable list of Researchers and removes non-matching nodes.
         * Used to provide results for Search.cshtml
         */
        private List<ResearcherModel> FindResearchersBy(string[] query)
        {
            List<ResearcherModel> result = new List<ResearcherModel>();

            if (query.Length == 0)
                // if it's empty then we don't need to do anything
                return result;

            // Now we loop through all of the researchers and attempt to match each string in the query to the researcher's name.
            // If a match is found, we add that researcher to our results and continue to the next researcher in the list.
            foreach (ResearcherModel AM in allResearchers)
            {
                if (result.Contains(AM))
                    continue;

                foreach (String S in query)
                {
                    if (String.IsNullOrWhiteSpace(S))
                        // If a string is empty it will probably match everything, so just ignore it
                        continue;

                    // Now we want to match every name in the string to an author
                    string[] sA = S.Split(' ');
                    if (sA.All(s => StringContains(AM.Author.Value.ToString(), s)))
                    {
                        result.Add(AM);
                        break;
                    }
                }
            }

            return result;
        }

        private List<ProgramModel> FindProgramsBy(string[] query)
        {
            List<ProgramModel> result = new List<ProgramModel>();

            if (query.Length == 0)
                // if it's empty then we don't need to do anything
                return result;

            // Now we loop through all of the programs and attempt to match each string in the query to the program's name and description.
            // If a match is found, we add that program to our results and continue to the next one in the list.
            foreach (ProgramModel PM in allPrograms)
            {
                if (result.Contains(PM))
                    continue;

                foreach (String S in query)
                {
                    if (String.IsNullOrWhiteSpace(S))
                        // If a string is empty it will probably match everything, so just ignore it
                        continue;

                    // Now we want to match every name in the string to a program name/description
                    string[] sA = S.Split(' ');
                    if (sA.All(s => StringContains(PM.Name, s) || StringContains(PM.Description, s)))
                    {
                        result.Add(PM);
                        break;
                    }
                }
            }

            return result;
        }
    }
}