using System;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;

namespace P5T.Models
{
    public class UrlModel
    {
        public int Id;
        public string Url;
        public string Description;

        public UrlModel(int id, string url, string description)
        {
            Id = id;
            Url = url;
            Description = description;
        }

        public static UrlModel Create(string url, string description)
        {
            // TODO : Create a UrlModel and return it.
            return DatabaseFacade.ExecuteTypedTx<UrlModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "INSERT INTO Url (url, url_description) OUTPUT INSERTED.url_id VALUES (@Url, @Description)";
                cmd.Parameters.Add("@Url", SqlDbType.NVarChar, 128).Value = url;
                cmd.Parameters.Add("@Description", SqlDbType.NVarChar, 128).Value = description;
                cmd.Prepare();
                return new UrlModel((Int32)cmd.ExecuteScalar(), url, description);
            });
        }

        public static UrlModel Find(int id)
        {
            // Find the UrlModel in the DB and return it.
            return DatabaseFacade.ExecuteTypedTx<UrlModel>(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "SELECT url, url_description FROM Url WHERE url_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();
                using (var rdr = cmd.ExecuteReader())
                {
                    if (rdr.Read())
                    {
                        return new UrlModel(id, rdr.GetString(rdr.GetOrdinal("url")), rdr.GetString(rdr.GetOrdinal("url_description")));
                    }
                    else
                    {
                        return null;
                    }
                }
            });
        }

        public static void Update(UrlModel model)
        {
            // Update the given UrlModel in the DB.
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "UPDATE Url SET url = @Url, url_description = @Description WHERE url_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = model.Id;
                cmd.Parameters.Add("@Url", SqlDbType.NVarChar, 128).Value = model.Url;
                cmd.Parameters.Add("Description", SqlDbType.NVarChar, 128).Value = model.Description;
                cmd.Prepare();
                return cmd.ExecuteNonQuery() == 1;
            });
        }

        public static void Delete(int id)
        {
            // Delete the given UrlModel from the DB.
            DatabaseFacade.ExecuteTx(delegate(SqlConnection conn, SqlTransaction tx, SqlCommand cmd)
            {
                cmd.CommandText = "DELETE FROM Url WHERE url_id = @Id";
                cmd.Parameters.Add("@Id", SqlDbType.Int).Value = id;
                cmd.Prepare();
                return cmd.ExecuteNonQuery() == 1;
            });
        }
    }
}