
-- Affiliation Tables

CREATE TABLE [dbo].[Affiliation] (
    [affiliation_id]    INT                 IDENTITY (1,1) NOT NULL,
    [title]             NVARCHAR (128)      NULL,
    [phone_number]      NCHAR (11)          NULL,
    [email_address]     NVARCHAR (128)           NULL,
    PRIMARY KEY CLUSTERED ([affiliation_id] ASC)
);

-- Primary Tables

CREATE TABLE [dbo].[Author] (
    [author_id]             INT                 IDENTITY (1, 1) NOT NULL,
    [affiliation_primary]   INT                 NOT NULL FOREIGN KEY REFERENCES [dbo].[Affiliation](affiliation_id),
    [first_name]            NVARCHAR (128)      NOT NULL,
    [last_name]             NVARCHAR (128)      NOT NULL,
    [middle_init]           NCHAR               NULL,
	PRIMARY KEY CLUSTERED ([author_id] ASC)
);

CREATE TABLE [dbo].[Researcher] (
    [user_id]               INT                 IDENTITY (1, 1) NOT NULL,
    [username]              NVARCHAR (32)       NOT NULL,
    [passhash]              NCHAR (128)         NOT NULL,
    [salt]                  NCHAR (128)         NOT NULL,
    [creation_datetime]     DATETIME            NOT NULL,
    [author_id]             INT                 NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id),
    [archival_date]         DATE                DEFAULT NULL,
    PRIMARY KEY CLUSTERED ([user_id] ASC),
    UNIQUE NONCLUSTERED ([username] ASC),
    UNIQUE NONCLUSTERED ([author_id] ASC)
);

CREATE TABLE [dbo].[Program] (
    [program_id]            INT             IDENTITY (1, 1) NOT NULL,
    [name]                  NVARCHAR (128)  NOT NULL,
    [program_description]   NVARCHAR (128)  NULL,
    [archival_date]         DATE            DEFAULT NULL,
    PRIMARY KEY CLUSTERED ([program_id] ASC)
);

CREATE TABLE [dbo].[Attachment] (
    [attachment_id]          INT             IDENTITY (1, 1) NOT NULL,
    [attachment_name]        NVARCHAR (128)  NOT NULL,
    [attachment_data]        VARBINARY (MAX)  NOT NULL,
    [attachment_description] NVARCHAR (128)  NULL,
    [prev_rev_id]            INT             NULL,
    PRIMARY KEY CLUSTERED ([attachment_id] ASC)
);

CREATE TABLE [dbo].[Url] (
    [url_id]                 INT                IDENTITY (1, 1) NOT NULL,
    [url]                    NVARCHAR (128)     NOT NULL,
    [url_description]        NVARCHAR (128)     NULL,
    PRIMARY KEY CLUSTERED ([url_id] ASC)
);

CREATE TABLE [dbo].[AuthorAffiliation] (
    [affiliation_id]    INT                 IDENTITY (1,1) NOT NULL FOREIGN KEY REFERENCES [dbo].[Affiliation](affiliation_id),
    [author_id]         INT                 NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id)
);

CREATE TABLE [dbo].[ProgramActivity] (
    [log_id] INT IDENTITY(1,1) NOT NULL,
    [date_time] DATETIME NOT NULL,
    [researcher_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [program_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [activity_type] INT NOT NULL,
    [param1_data] NVARCHAR(128) NULL,
    [param2_data] NVARCHAR(128) NULL
    PRIMARY KEY CLUSTERED ([log_id] ASC)
);

CREATE TABLE [dbo].[ProgramNotification] (
    [notification_id] INT IDENTITY(1,1) NOT NULL,
    [user_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id),
    [log_id] INT NOT NULL REFERENCES [dbo].[ProgramActivity](log_id) ON DELETE NO ACTION
    PRIMARY KEY CLUSTERED ([notification_id] ASC)
);

-- Research Article Tables

CREATE TABLE [dbo].[ResearchArticle] (
    [research_article_id]            INT                IDENTITY (1,1) NOT NULL,
    [program_id]                     INT                NOT NULL FOREIGN KEY REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [research_article_description]   NVARCHAR (128)     NULL,
    [archival_date]                  DATE               DEFAULT NULL,
    PRIMARY KEY CLUSTERED ([research_article_id] ASC)
);

CREATE TABLE [dbo].[ResearchArticleAuthor] (
    [research_article_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[ResearchArticle](research_article_id) ON DELETE CASCADE,
    [author_id]                     INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id) ON DELETE CASCADE,
    [author_order]                  INT         NOT NULL
);

CREATE TABLE [dbo].[ResearchArticleAttachment] (
    [research_article_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[ResearchArticle](research_article_id) ON DELETE CASCADE,
    [attachment_id]                 INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Attachment](attachment_id) ON DELETE CASCADE,
    [attachment_order]              INT         NOT NULL
);

CREATE TABLE [dbo].[ResearchArticleURL] (
    [research_article_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[ResearchArticle](research_article_id) ON DELETE CASCADE,
    [url_id]                        INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Url](url_id) ON DELETE CASCADE,
    [url_order]                     INT         NOT NULL
);

CREATE TABLE [dbo].[ResearchArticleActivity] (
    [log_id] INT IDENTITY(1,1) NOT NULL,
    [date_time] DATETIME NOT NULL,
    [researcher_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [research_article_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[ResearchArticle](research_article_id) ON DELETE CASCADE,
    [activity_type] INT NOT NULL,
    [param1_data] NVARCHAR(128) NULL,
    [param2_data] NVARCHAR(128) NULL
    PRIMARY KEY CLUSTERED ([log_id] ASC)
);

CREATE TABLE [dbo].[ResearchArticleNotification] (
    [notification_id] INT IDENTITY(1,1) NOT NULL,
    [user_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id),
    [log_id] INT NOT NULL REFERENCES [dbo].[ResearchArticleActivity](log_id) ON DELETE CASCADE
    PRIMARY KEY CLUSTERED ([notification_id] ASC)
);

-- Conference Presentation Tables

CREATE TABLE [dbo].[ConferencePresentation] (
    [presentation_id]				INT					IDENTITY (1, 1) NOT NULL,
    [program_id]                     INT                NOT NULL FOREIGN KEY REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [presentation_title]             NVARCHAR (50)      NOT NULL,
    [presentation_description]       NVARCHAR (256)     NOT NULL,
    [presentation_type]              NVARCHAR (64)      NOT NULL,
    [conference_name]                NVARCHAR (128)     NOT NULL,
    [conference_location]            NVARCHAR (128)     NOT NULL,
    [conference_date]                DATETIME           NOT NULL,
    [archival_date]                  DATE               DEFAULT NULL,
    PRIMARY KEY CLUSTERED ([presentation_id] ASC)
);

CREATE TABLE [dbo].[ConferencePresentationAuthor] (
    [presentation_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[ConferencePresentation](presentation_id) ON DELETE CASCADE,
    [author_id]                     INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id) ON DELETE CASCADE,
    [author_order]                  INT         NOT NULL
);

CREATE TABLE [dbo].[ConferencePresentationAttachment] (
    [presentation_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[ConferencePresentation](presentation_id) ON DELETE CASCADE,
    [attachment_id]                 INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Attachment](attachment_id) ON DELETE CASCADE,
    [attachment_order]              INT         NOT NULL
);

CREATE TABLE [dbo].[ConferencePresentationUrl] (
    [presentation_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[ConferencePresentation](presentation_id) ON DELETE CASCADE,
    [url_id]                        INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Url](url_id) ON DELETE CASCADE,
    [url_order]                     INT         NOT NULL
);

CREATE TABLE [dbo].[ConferencePresentationActivity] (
    [log_id] INT IDENTITY(1,1) NOT NULL,
    [date_time] DATETIME NOT NULL,
    [researcher_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [presentation_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[ConferencePresentation](presentation_id) ON DELETE CASCADE,
    [activity_type] INT NOT NULL,
    [param1_data] NVARCHAR(128) NULL,
    [param2_data] NVARCHAR(128) NULL,
    PRIMARY KEY CLUSTERED ([log_id] ASC)
);

CREATE TABLE [dbo].[ConferencePresentationNotification] (
    [notification_id] INT IDENTITY(1,1) NOT NULL,
    [user_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id),
    [log_id] INT NOT NULL REFERENCES [dbo].[ConferencePresentationActivity](log_id) ON DELETE CASCADE
    PRIMARY KEY CLUSTERED ([notification_id] ASC)
);

-- Journal Article Tables

CREATE TABLE [dbo].[JournalArticle] (
    [journal_article_id]             INT                IDENTITY (1,1) NOT NULL,
    [program_id]                     INT                NOT NULL FOREIGN KEY REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [journal_article_description]    NVARCHAR (128)     NULL,
    [archival_date]                  DATE               DEFAULT NULL,
    PRIMARY KEY CLUSTERED ([journal_article_id] ASC)
);

CREATE TABLE [dbo].[JournalArticleAuthor] (
    [journal_article_id]            INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[JournalArticle](journal_article_id) ON DELETE CASCADE,
    [author_id]                     INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id) ON DELETE CASCADE,
    [author_order]                  INT         NOT NULL
);

CREATE TABLE [dbo].[JournalArticleAttachment] (
    [journal_article_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[JournalArticle](journal_article_id) ON DELETE CASCADE,
    [attachment_id]                 INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Attachment](attachment_id) ON DELETE CASCADE,
    [attachment_order]              INT         NOT NULL
);

CREATE TABLE [dbo].[JournalArticleUrl] (
    [journal_article_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[JournalArticle](journal_article_id) ON DELETE CASCADE,
    [url_id]                        INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Url](url_id) ON DELETE CASCADE,
    [url_order]                     INT         NOT NULL
);

CREATE TABLE [dbo].[JournalArticleActivity] (
    [log_id] INT IDENTITY(1,1) NOT NULL,
    [date_time] DATETIME NOT NULL,
    [researcher_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [journal_article_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[JournalArticle](journal_article_id) ON DELETE CASCADE,
    [activity_type] INT NOT NULL,
    [param1_data] NVARCHAR(128) NULL,
    [param2_data] NVARCHAR(128) NULL
    PRIMARY KEY CLUSTERED ([log_id] ASC)
);

CREATE TABLE [dbo].[JournalArticleNotification] (
    [notification_id] INT IDENTITY(1,1) NOT NULL,
    [user_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id),
    [log_id] INT NOT NULL REFERENCES [dbo].[JournalArticleActivity](log_id) ON DELETE CASCADE
    PRIMARY KEY CLUSTERED ([notification_id] ASC)
);

-- Media Interview Tables

CREATE TABLE [dbo].[MediaInterview] (
    [interview_id]                   INT                IDENTITY (1, 1) NOT NULL,
    [program_id]                     INT                NOT NULL FOREIGN KEY REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [interview_description]          NVARCHAR (128)     NULL,
    [interview_topic]                NVARCHAR(128)      NOT NULL,
    [interview_outlet]               NVARCHAR(64)       NOT NULL,
    [interview_location]             NVARCHAR(128)      NOT NULL,
    [interview_date]                 DATETIME           NOT NULL,
    [archival_date]                  DATE               DEFAULT NULL,
	PRIMARY KEY CLUSTERED ([interview_id] ASC)
);

CREATE TABLE [dbo].[MediaInterviewAuthor] (
    [interview_id]            INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[MediaInterview](interview_id) ON DELETE CASCADE,
    [author_id]                     INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id) ON DELETE CASCADE,
    [author_order]                  INT         NOT NULL
);

CREATE TABLE [dbo].[MediaInterviewAttachment] (
    [interview_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[MediaInterview](interview_id) ON DELETE CASCADE,
    [attachment_id]                 INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Attachment](attachment_id) ON DELETE CASCADE,
    [attachment_order]              INT         NOT NULL
);

CREATE TABLE [dbo].[MediaInterviewUrl] (
    [interview_id]           INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[MediaInterview](interview_id) ON DELETE CASCADE,
    [url_id]                        INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Url](url_id) ON DELETE CASCADE,
    [url_order]                     INT         NOT NULL
);

CREATE TABLE [dbo].[MediaInterviewActivity] (
    [log_id] INT IDENTITY(1,1) NOT NULL,
    [date_time] DATETIME NOT NULL,
    [researcher_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [interview_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[MediaInterview](interview_id) ON DELETE CASCADE,
    [activity_type] INT NOT NULL,
    [param1_data] NVARCHAR(128) NULL,
    [param2_data] NVARCHAR(128) NULL
    PRIMARY KEY CLUSTERED ([log_id] ASC)
);

CREATE TABLE [dbo].[MediaInterviewNotification] (
    [notification_id] INT IDENTITY(1,1) NOT NULL,
    [user_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id),
    [log_id] INT NOT NULL REFERENCES [dbo].[MediaInterviewActivity](log_id) ON DELETE CASCADE
    PRIMARY KEY CLUSTERED ([notification_id] ASC)
);

-- Other Attachment/Url Tables

CREATE TABLE [dbo].[Other] (
    [other_id]            INT                   IDENTITY (1,1) NOT NULL,
    [program_id]          INT                   NOT NULL FOREIGN KEY REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [other_description]   NVARCHAR (128)        NULL,
    [archival_date]       DATE                  DEFAULT NULL,
    PRIMARY KEY CLUSTERED ([other_id] ASC)
);

CREATE TABLE [dbo].[OtherAuthor] (
    [other_id]                      INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Other](other_id) ON DELETE CASCADE,
    [author_id]                     INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id) ON DELETE CASCADE,
    [author_order]                  INT         NOT NULL
);

CREATE TABLE [dbo].[OtherAttachment] (
    [other_id]                      INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Other](other_id) ON DELETE CASCADE,
    [attachment_id]                 INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Attachment](attachment_id) ON DELETE CASCADE,
    [attachment_order]              INT         NOT NULL
);

CREATE TABLE [dbo].[OtherUrl] (
    [other_id]                      INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Other](other_id) ON DELETE CASCADE,
    [url_id]                        INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Url](url_id) ON DELETE CASCADE,
    [url_order]                     INT         NOT NULL
);

CREATE TABLE [dbo].[OtherActivity] (
    [log_id] INT IDENTITY(1,1) NOT NULL,
    [date_time] DATETIME NOT NULL,
    [researcher_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [other_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Other](other_id) ON DELETE CASCADE,
    [activity_type] INT NOT NULL,
    [param1_data] NVARCHAR(128) NULL,
    [param2_data] NVARCHAR(128) NULL
    PRIMARY KEY CLUSTERED ([log_id] ASC)
);

CREATE TABLE [dbo].[OtherNotification] (
    [notification_id] INT IDENTITY(1,1) NOT NULL,
    [user_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id),
    [log_id] INT NOT NULL REFERENCES [dbo].[OtherActivity](log_id) ON DELETE CASCADE
    PRIMARY KEY CLUSTERED ([notification_id] ASC)
);

-- Grant Tables

CREATE TABLE [dbo].[Grant] (
    [grant_id]            INT                   IDENTITY(1,1)      NOT NULL,
    [program_id]          INT                   NOT NULL FOREIGN KEY REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [grant_description]   NVARCHAR (128)        NULL,
    [grant_source]        NVARCHAR (128)        NOT NULL,
    [grant_number]        NVARCHAR (128)        NOT NULL, -- Assigned by granting institution, might not truly be unique on a global scale
    [grant_title]         NVARCHAR (128)        NOT NULL, 
    [grant_start]         DATE                  NOT NULL,
    [grant_end]           DATE                  NOT NULL,
    [archival_date]       DATE                  DEFAULT NULL,
	PRIMARY KEY CLUSTERED ([grant_id] ASC)
);

CREATE TABLE [dbo].[GrantAuthor] (
    [grant_id]                      INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Grant](grant_id) ON DELETE CASCADE,
    [author_id]                     INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Author](author_id) ON DELETE CASCADE,
    [author_order]                  INT         NOT NULL
);

CREATE TABLE [dbo].[GrantAttachment] (
    [grant_id]                      INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Grant](grant_id) ON DELETE CASCADE,
    [attachment_id]                 INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Attachment](attachment_id) ON DELETE CASCADE,
    [attachment_order]              INT         NOT NULL
);

CREATE TABLE [dbo].[GrantUrl] (
    [grant_id]                      INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Grant](grant_id) ON DELETE CASCADE,
    [url_id]                        INT         NOT NULL FOREIGN KEY REFERENCES [dbo].[Url](url_id) ON DELETE CASCADE,
    [url_order]                     INT         NOT NULL
);

CREATE TABLE [dbo].[GrantActivity] (
    [log_id] INT IDENTITY(1,1) NOT NULL,
    [date_time] DATETIME NOT NULL,
    [researcher_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [grant_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Grant](grant_id) ON DELETE CASCADE,
    [activity_type] INT NOT NULL,
    [param1_data] NVARCHAR(128) NULL,
    [param2_data] NVARCHAR(128) NULL
    PRIMARY KEY CLUSTERED ([log_id] ASC)
);

CREATE TABLE [dbo].[GrantNotification] (
    [notification_id] INT IDENTITY(1,1) NOT NULL,
    [user_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id),
    [log_id] INT NOT NULL REFERENCES [dbo].[GrantActivity](log_id) ON DELETE CASCADE
    PRIMARY KEY CLUSTERED ([notification_id] ASC)
);

CREATE TABLE [dbo].[ProgramSubscriptions] (
    [sub_id] INT IDENTITY(1,1) NOT NULL,
    [program_id] INT NOT NULL REFERENCES [dbo].[Program](program_id) ON DELETE CASCADE,
    [subscriber_id] INT NOT NULL REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE
    PRIMARY KEY CLUSTERED ([sub_id] ASC)
);

CREATE TABLE [dbo].[ResearcherSubscriptions] (
    [sub_id] INT IDENTITY(1,1) NOT NULL,
    [subscribee_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE CASCADE,
    [subscriber_id] INT NOT NULL FOREIGN KEY REFERENCES [dbo].[Researcher](user_id) ON DELETE NO ACTION
    PRIMARY KEY CLUSTERED ([sub_id] ASC)
);

-- Default Data (root with password root)

INSERT INTO [dbo].[Affiliation] ([title], [phone_number], [email_address]) VALUES ('The Admin', '1235550123', 'admin@admin.com');
INSERT INTO [dbo].[Author] ([affiliation_primary], [first_name], [last_name], [middle_init]) VALUES (1, 'John', 'Doe', NULL);
INSERT INTO [dbo].[Researcher] ([author_id], [username], [passhash], [salt], [creation_datetime]) VALUES (1, 'root', 'A9C65AE95C3B81755C754DA79EEFF35ABF273B243BB4456AA45E220CFA4C12E1DACDD16D4ECB45FA748B5A2576E59806EA011F7ADFFDF5A3D2BC0DE2B21E64E0', '61616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161', GETDATE());
