
import os
import fnmatch

def truncate(f, n):
    '''Truncates/pads a float f to n decimal places without rounding'''
    s = '{}'.format(f)
    if 'e' in s or 'E' in s:
        return '{0:.{1}f}'.format(f, n)
    i, p, d = s.partition('.')
    return '.'.join([i, (d+'0'*n)[:n]])

def is_space(c): 
    return c == ' ' or c == '\t' or c == '\r'

def is_newline(c):
    return c == '\n'

STATE_EMPTY = 0
STATE_SOURCE = 1

STATE_MAYBE_COMMENT = 2

STATE_LINE_COMMENT = 3

STATE_MULTI_COMMENT = 4
STATE_MULTI_COMMENT2 = 5

STATE_S_QUOTE = 6
STATE_D_QUOTE = 7

class MetricsResult:
    __slots__ = ('totalLines', 
                 'sourceLines', 
                 'commentLines', 
                 'emptyLines')

    def __init__(self):
        self.totalLines = 0
        self.sourceLines = 0
        self.commentLines = 0
        self.emptyLines = 0

def parseMetrics(data):
    mr = MetricsResult()

    state = STATE_EMPTY
    i = 0
    N = len(data)
    while i < N:
        c = data[i]
        if state == STATE_EMPTY:
            if is_newline(c):
                mr.totalLines += 1
                mr.emptyLines += 1
                i += 1
            elif not is_space(c):
                state = STATE_SOURCE
            else:
                i += 1
        elif state == STATE_SOURCE:
            if is_newline(c):
                state = STATE_EMPTY
                mr.totalLines += 1
                mr.sourceLines += 1
                i += 1
            elif c == '"':
                state = STATE_D_QUOTE
                i += 1
            elif c == '\'':
                state = STATE_S_QUOTE
                i += 1
            elif c == '/':
                state = STATE_MAYBE_COMMENT
                i += 1
            else:
                i += 1
        elif state == STATE_D_QUOTE:
            if is_newline(c):
                mr.sourceLines += 1
                mr.totalLines += 1
                i += 1
            elif c == '\\':
                i += 2
            elif c == '"':
                state = STATE_SOURCE
                i += 1
            else:
                i += 1
        elif state == STATE_S_QUOTE:
            if is_newline(c):
                mr.sourceLines += 1
                mr.totalLines += 1
                i += 1
            elif c == '\\':
                i += 2
            elif c == '\'':
                state = STATE_SOURCE
                i += 1
            else:
                i += 1
        elif state == STATE_MAYBE_COMMENT:
            if c == '/':
                state = STATE_LINE_COMMENT
                i += 1
            elif c == '*':
                state = STATE_MULTI_COMMENT
                i += 1
            else:
                state = STATE_SOURCE
        elif state == STATE_LINE_COMMENT:
            if is_newline(c):
                mr.commentLines += 1
                mr.totalLines += 1
                i += 1
            else:
                i += 1
        elif state == STATE_MULTI_COMMENT:
            if is_newline(c):
                mr.commentLines += 1
                mr.totalLines += 1
                i += 1
            elif c == '*':
                state = STATE_MULTI_COMMENT2
                i += 1
            else:
                i += 1
        elif state == STATE_MUTLI_COMMENT2:
            if c == '/':
                state = STATE_SOURCE
                i += 1
            else:
                state = STATE_MULTI_COMMENT
        else:
            raise RuntimeError("Invalid state!")
    return mr

def printMetricResult(name, mr):
    print(name)
    print("\tSource Lines: ", mr.sourceLines, "/", mr.totalLines, " (", truncate(100 * mr.sourceLines / mr.totalLines, 2) , "%)", sep='')
    print("\tComment Lines: ", mr.commentLines,"/", mr.totalLines, " (", truncate(100 * mr.commentLines / mr.totalLines, 2) , "%)", sep='')
    print("\tEmpty Lines: ", mr.emptyLines, "/", mr.totalLines , " (", truncate(100 * mr.emptyLines / mr.totalLines, 2) , "%)", sep='')
    print()

def main(argv):
    if len(argv) < 2:
        path = "."
    else:
        path = argv[1]

    if not os.path.isdir(path):
        raise runtime_error("Expected root directory as argument to script.")

    overallMetric = MetricsResult()
    for root, _, filenames in os.walk(path):
        for filename in fnmatch.filter(filenames, '*.cs'):
            with open(filename) as f:
                metricResult = parseMetrics(f.read())
                
                printMetricResult("Filename: " + filename, metricResult)

                overallMetric.totalLines += metricResult.totalLines
                overallMetric.sourceLines += metricResult.sourceLines
                overallMetric.commentLines += metricResult.commentLines
                overallMetric.emptyLines += metricResult.emptyLines

    print("=============================================")
    printMetricResult("Overall", overallMetric)


if __name__ == "__main__":
    import sys
    main(sys.argv)
